package db.handler;

import db.DBApiEntry;
import db.Point;
import db.PointVals;
import db.entity.DataPackage;
import db.entity.LogItem;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import db.service.LogFileService;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.util.HashMap;

@Component
@Slf4j
public class DataPackageHandler {

    @Value("${reso-db.metric-name}")
    private String metricName;

    @Autowired
    private DBApiEntry entry;

    @Autowired
    private LogFileService packetFileMetaService;

    public void handlerDataPackage(DataPackage dataPackage) throws IOException {
        log.info("开始保存数据包{}", dataPackage);

        log.info("开始保存至磁盘");
        // 封装 LogItem
        LogItem logItem = new LogItem((short) 0x55, dataPackage.getTimestamp(), dataPackage.getData());

        long longIndex = packetFileMetaService.appendLog(logItem);
        double index = Double.longBitsToDouble(longIndex);
        log.info("写入磁盘成功");

        // 写入数据库
        log.info("开始尝试插入ResoDB，longIndex={}，doubleIndex={}", longIndex, index);

        HashMap<String, String> tags = new HashMap<>();
        tags.put("sMac", dataPackage.getSMac());
        tags.put("dMac", dataPackage.getDMac());
        tags.put("sPort", dataPackage.getSPort());
        tags.put("dPort", dataPackage.getDPort());
        tags.put("sIp", dataPackage.getSIp());
        tags.put("dIp", dataPackage.getDIp());
        tags.put("probeId", String.valueOf(dataPackage.getProbeId()));
        Point point = new Point(metricName, tags);

        long[] utcTimes = new long[1];
        double[] values = new double[1];

        utcTimes[0] = dataPackage.getTimestamp();
        values[0] = index;
        PointVals pointVals = new PointVals(point, 1, utcTimes, values);
        int insertCount = entry.insertSinglPoint(pointVals);
        log.info("插入ResoDB结束，point={}，插入{}条", pointVals, insertCount);
    }


}
