package db.netty;

import db.Globals;
import db.entity.BasePackage;
import db.entity.DataPackage;
import db.entity.ListenPackage;
import db.util.ConvertUtils;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.codec.MessageToMessageDecoder;
import lombok.extern.slf4j.Slf4j;

import java.util.Arrays;
import java.util.List;

@Slf4j
public class Decoder extends MessageToMessageDecoder<DatagramPacket> {

    @Override
    protected void decode(ChannelHandlerContext ctx, DatagramPacket datagramPacket, List<Object> out) throws Exception {
        ByteBuf in = datagramPacket.content();
        byte[] bytesReady = new byte[in.readableBytes()];
        in.getBytes(0, bytesReady);
        log.info("解码器收到数据：{} 从 {}", ConvertUtils.bytes2HexArray(bytesReady), ctx);

        int magic = in.getInt(0);

        if (magic != Globals.PROBE_MAGIC) {
            log.warn("魔数错误，消息为：{}", ConvertUtils.bytes2HexArray(bytesReady));
            return;
        }

        int ver = in.getInt(4);
        short operationType = in.getShort(8);
        short responseCode = in.getShort(10);
        int packageId = in.getInt(12);
        short operationCode = in.getShort(12);
        short packageCode = in.getShort(14);
        short packageLength = in.getShort(16);

        BasePackage basePackage;


        if (operationType == 0x0003 && operationCode == 0x0006) {
            log.info("解码器转换为数据报文");
            basePackage = new DataPackage();
        } else if (operationType == 0x0001 && (operationCode == 0x0004 || operationCode == 0x0005)) {
            // 发送的监听开始
            log.info("解码器转换为监听报文");
            basePackage = new ListenPackage();
        } else {
            log.error("解码器无法识别报文，operationType={}，operationCode={}", operationType, operationCode);
            return;
        }

        // 把byteBuf中的内容填充到包对象中
        basePackage.setHeader(in);
        basePackage.setPayload(in);

        basePackage.setFromHost(datagramPacket.sender().getHostString());
        basePackage.setFromPort(datagramPacket.sender().getPort());

        out.add(basePackage);
        log.info("解码器转码为：{}", basePackage);
    }
}


