package db.netty;

import db.Globals;
import db.entity.BasePackage;
import db.entity.DataPackage;
import db.entity.ListenPackage;
import db.util.ContentUtils;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;
import lombok.extern.slf4j.Slf4j;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;


@Slf4j
public class NettyClientHandler extends SimpleChannelInboundHandler<BasePackage> {

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, BasePackage basePackage) throws Exception {
        System.out.println("收到数据:" + basePackage);
        if (basePackage instanceof ListenPackage) {
            sendDataPackage(ctx, basePackage.getFromHost(), basePackage.getFromPort());
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        cause.printStackTrace();
        ctx.close();
    }

    public void sendDataPackage(ChannelHandlerContext ctx, String toHost, int toPort) throws ParseException,
            InterruptedException {
        // 以下固定不变
        int version = Globals.VERSION;
        short operationType = 0x3;
        short responseCode = 200;
        short operationCode = 0x0006;
        short packageCode = 0;
        short probeId = 0x0010;
        byte port = 0x01;
        byte dataStrategy = 0;
        byte reserve = 0;
        byte clockSync = (byte) 0xAA;

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date startDate = format.parse("2022-12-01 00:00:00");
        Calendar cal = Calendar.getInstance();
        cal.setTime(startDate);


        for (int i = 0; i < 2; i++) {
            short year = (short) cal.get(Calendar.YEAR);
            short day = (short) cal.get(Calendar.DAY_OF_YEAR);
            short hour = (short) cal.get(Calendar.HOUR_OF_DAY);
            short minute = (short) cal.get(Calendar.MINUTE);
            short second = (short) cal.get(Calendar.SECOND);
            int microSecond = cal.get(Calendar.MILLISECOND) * 1000;
            byte[] data = ContentUtils.generateData(0x000002555559L, 0x000002444455L, "192.168.1.1",
                    "192.168.31.1", (short) 8080, (short) 9090);
            DataPackage dataPackage = new DataPackage(version, operationType, responseCode, operationCode,
                    packageCode, probeId, port, dataStrategy, reserve, clockSync, year, day, hour, minute, second,
                    microSecond, data);
            log.info("第{}个dataPackage：{}", i, dataPackage);
            dataPackage.setToHostPort(toHost, toPort);
            send(ctx, dataPackage);
            Thread.sleep(10);
            cal.add(Calendar.SECOND, 1);
        }
        log.info("发送完成");
    }

    public void send(ChannelHandlerContext ctx, BasePackage basePackage) {
        ctx.writeAndFlush(basePackage);
    }
}
