package db.netty;

import db.entity.BasePackage;
import db.entity.ListenPackage;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.DatagramPacket;
import io.netty.channel.socket.nio.NioDatagramChannel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.DependsOn;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.net.InetSocketAddress;

@Component
@DependsOn("springUtils")
@Slf4j
public class NettyServer {

    EventLoopGroup group = new NioEventLoopGroup();

    @Value("${netty.port}")
    private Integer port;

    private Channel channel;

    /**
     * 启动Netty Server
     *
     * @throws InterruptedException
     */
    @PostConstruct
    public void start() throws InterruptedException {
        log.info("开始启动 Netty Server，port: {}", port);
        Bootstrap bootstrap = new Bootstrap();
        bootstrap.group(group)
                // 指定Channel
                .channel(NioDatagramChannel.class)

                .option(ChannelOption.SO_BROADCAST, true)

                .handler(new ChannelInitializer<NioDatagramChannel>() {
                    @Override
                    public void initChannel(final NioDatagramChannel ch) throws Exception {
                        ChannelPipeline p = ch.pipeline();
                        p.addLast(new Encoder(), new Decoder());
                        p.addLast(new NettyServerHandler());
                    }
                });


        ChannelFuture future = bootstrap.bind(port).sync();
        if (future.isSuccess()) {
            log.info("成功启动 Netty Server，port: {}", port);
            channel = future.channel();
        } else {
            log.error("失败启动 Netty Server，port: {}", port);
        }
    }

    public void send(BasePackage basePackage) {
        channel.writeAndFlush(basePackage);
    }

    @PreDestroy
    public void destroy() {
        group.shutdownGracefully();
        log.info("关闭Netty");
    }
}
